﻿using NUnit.Framework;
using FluentAssertions;

using System;
using System.Collections.Generic;
using System.Text;

using Encompass;

namespace Tests
{
    public class WorldTest
    {
        struct TestComponent : IComponent { }
        struct TestDrawComponent : IComponent, IDrawComponent { }

        static List<object> drawOrder = new List<object>();

        class TestEntityRenderer : OrderedRenderer<TestDrawComponent>
        {
            public override void Render(Entity entity, TestDrawComponent testDrawComponent)
            {
                drawOrder.Add(entity);
            }
        }

        class TestGeneralRenderer : GeneralRenderer
        {
            public override void Render()
            {
                drawOrder.Add(this);
            }
        }

        [Test]
        public void DrawOrder()
        {
            var worldBuilder = new WorldBuilder();
            worldBuilder.AddOrderedRenderer(new TestEntityRenderer());
            var testGeneralRenderer = worldBuilder.AddGeneralRenderer(new TestGeneralRenderer(), 7);

            TestComponent testComponent;
            TestDrawComponent testDrawComponent = default(TestDrawComponent);

            var entity = worldBuilder.CreateEntity();
            worldBuilder.SetComponent(entity, testComponent);
            worldBuilder.SetDrawComponent(entity, testDrawComponent, 3);

            TestDrawComponent testDrawComponentTwo = default(TestDrawComponent);

            var entityTwo = worldBuilder.CreateEntity();
            worldBuilder.SetComponent(entityTwo, testComponent);
            worldBuilder.SetDrawComponent(entityTwo, testDrawComponentTwo, 1);

            TestDrawComponent testDrawComponentThree = default(TestDrawComponent);

            var entityThree = worldBuilder.CreateEntity();
            worldBuilder.SetComponent(entityThree, testComponent);
            worldBuilder.SetDrawComponent(entityThree, testDrawComponentThree, 5);

            TestDrawComponent testDrawComponentFour = default(TestDrawComponent);

            var entityFour = worldBuilder.CreateEntity();
            worldBuilder.SetComponent(entityFour, testComponent);
            worldBuilder.SetDrawComponent(entityFour, testDrawComponentFour, -5);

            var world = worldBuilder.Build();

            world.Update(0.01f);
            world.Draw();

            drawOrder.Should().BeEquivalentTo(entityFour, entityTwo, entity, entityThree, testGeneralRenderer);
        }
    }
}
